@@ -33,14 +33,14 @@
 #include <ucommon/memory.h>
 #endif
 
-#ifndef	_UCOMMON_SOCKET_H_
-#include <ucommon/socket.h>
-#endif
-
 #ifndef	_UCOMMON_TIMERS_H_
 #include <ucommon/timers.h>
 #endif
 
+#ifndef	_UCOMMON_SOCKET_H_
+#include <ucommon/socket.h>
+#endif
+
 #include <stdio.h>
 #include <string.h>
 #include <stdarg.h>
@@ -53,6 +53,8 @@ NAMESPACE_UCOMMON
  */
 typedef	unsigned short strsize_t;
 
+class StringFormat;
+
 /**
  * A copy-on-write string class that operates by reference count.  This string
  * class anchors a counted object that is managed as a copy-on-write
@@ -66,6 +68,8 @@ typedef	unsigned short strsize_t;
 class __EXPORT string : public Object
 {
 protected:
+	friend class StringFormat;
+
 	/**
 	 * This is an internal class which contains the actual string data
 	 * along with some control fields.  The string can be either NULL
@@ -119,6 +123,12 @@ protected:
 		 */
 		void set(strsize_t offset, const char *text, strsize_t size);
 
+		/**
+		 * Set our string from a string formatting object.
+		 * @param format object to set from.
+		 */
+		void set(const StringFormat& format);
+
 		/**
 		 * Set our string from null terminated text up to our allocated size.
 		 * @param text to set from.
@@ -131,6 +141,12 @@ protected:
 		 */
 		void add(const char *text);
 
+		/**
+		 * Append text to string buffer from formatting object.
+		 * @param formatting object to add from.
+		 */
+		void add(const StringFormat &format);
+
 		/**
 		 * Append a single character to our string buffer.
 		 * @param character to append.
@@ -182,6 +198,13 @@ protected:
 	 */
 	virtual int compare(const char *string) const;
 
+	/**
+	* Test if two string values are equal.
+	* @param string to compare with.
+	* @return true if equal.
+	*/
+	bool equal(const char *string) const;
+
 	/**
 	 * Increase retention of our reference counted cstring.  May be overriden
 	 * for memstring which has fixed cstring object.
@@ -208,6 +231,8 @@ protected:
 	 */
 	virtual void cow(strsize_t size = 0);
 
+	strsize_t getStringSize(void);
+
 public:
 	/**
 	 * A constant for an invalid position value.
@@ -219,6 +244,12 @@ public:
 	 */
 	string();
 
+	/**
+	 * Create a string from a formatting object.
+	 * @param format object to use in creating string.
+	 */
+	string(const StringFormat& format);
+
 	/**
 	 * Create an empty string with a buffer pre-allocated to a specified size.
 	 * @param size of buffer to allocate.
@@ -337,7 +368,13 @@ public:
 	 * @param text string to set.
 	 */
 	void set(const char *text);
-	
+
+	/**
+	 * Set string object to text of a formatted object.
+	 * @param format object to use.
+	 */
+	void set(const StringFormat& format);
+
 	/**
 	 * Set a portion of the string object at a specified offset to a text
 	 * string.
@@ -365,6 +402,13 @@ public:
 	 */
 	void rset(const char *text, char overflow, strsize_t offset, strsize_t size = 0);
 
+	/**
+	 * Append formatted object to our string buffer.
+	 * @param format object to append.
+	 */
+	void add(const StringFormat& format);
+
+
 	/**
 	 * Append null terminated text to our string buffer.
 	 * @param text to append.
@@ -645,6 +689,13 @@ public:
 	 */
 	string& operator^=(const string& object);
 
+	/**
+	 * Create new cow instance and assign value from formatted string object.
+	 * @param format object to assign from.
+	 * @return our object for expression use.
+	 */
+	string& operator^=(const StringFormat& format);
+
 	/**
 	 * Create new cow instance and assign value from null terminated text.
 	 * @param text to assign from.
@@ -659,6 +710,13 @@ public:
 	 */
 	string& operator+(const char *text);
 
+	/**
+	 * Concatenate string formatted object to our object.  This creates a new
+	 * copy-on-write instance to hold the concatenated string.
+	 * @param format object to concatenate.
+	 */
+	string& operator+(const StringFormat& format);
+
 	/**
 	 * Concatenate null terminated text to our object.  This directly
 	 * appends the text to the string buffer and does not resize the
@@ -667,6 +725,14 @@ public:
 	 */
 	string& operator&(const char *text);
 
+	/**
+	 * Concatenate formatted string object to our object.  This directly
+	 * appends the text to the string buffer and does not resize the
+	 * object if the existing cstring allocation space is fully used.
+	 * @param format object to concatenate.
+	 */
+	string& operator&(const StringFormat& format);
+
 	/**
 	 * Assign our string with the cstring of another object.  If we had
 	 * an active string reference, it is released.  The object now has
@@ -675,6 +741,12 @@ public:
 	 */
 	string& operator=(const string& object);
 
+	/**
+	 * Assign our string from a string formatting object.
+	 * @param format object to assign from.
+	 */
+	string& operator=(const StringFormat& format);
+
 	/**
 	 * Assign text to our existing buffer.  This performs a set method.
 	 * @param text to assign from.
@@ -986,6 +1058,14 @@ public:
 	 */
 	static int compare(const char *text1, const char *text2);
 
+	/**
+	 * Simple equal test for strings.
+	 * @param text1 to test.
+	 * @param text2 to test.
+	 * @return true if equal and case is same.
+	 */
+	static bool equal(const char *text1, const char *text2);
+
 	/**
 	 * Safe string comparison function.
 	 * @param text1 to compare.
@@ -995,13 +1075,30 @@ public:
 	 */
 	static int compare(const char *text1, const char *text2, size_t size);
 
+	/**
+	 * Simple equal test for strings.
+	 * @param text1 to test.
+	 * @param text2 to test.
+	 * @param size limit of strings to compare.
+	 * @return true if equal and case is same.
+	 */
+	static bool equal(const char *text1, const char *text2, size_t size);
+
 	/**
 	 * Safe case insensitive string comparison function.
 	 * @param text1 to compare.
 	 * @param text2 to compare.
 	 * @return 0 if equal, >0 if text1 > text2, <0 if text1 < text2.
 	 */
-	static int icompare(const char *text1, const char *text2);
+	static int case_compare(const char *text1, const char *text2);
+
+	/**
+	 * Simple case insensitive equal test for strings.
+	 * @param text1 to test.
+	 * @param text2 to test.
+	 * @return true if equal.
+	 */
+	static bool case_equal(const char *text1, const char *text2);
 
 	/**
 	 * Safe case insensitive string comparison function.
@@ -1010,7 +1107,16 @@ public:
 	 * @param size limit of strings to compare.
 	 * @return 0 if equal, >0 if text1 > text2, <0 if text1 < text2.
 	 */
-	static int icompare(const char *text1, const char *text2, size_t size);
+	static int case_compare(const char *text1, const char *text2, size_t size);
+
+	/**
+	 * Simple case insenstive equal test for strings.
+	 * @param text1 to test.
+	 * @param text2 to test.
+	 * @param size limit of strings to compare.
+	 * @return true if equal.
+	 */
+	static bool case_equal(const char *text1, const char *text2, size_t size);
 
 	/**
 	 * Return start of string after characters to trim from beginning.
@@ -1364,6 +1470,33 @@ public:
 	static memstring *create(mempager *pager, strsize_t size, char fill = 0);
 };
 
+/**
+ * A string conversion class for use as a base class in objects which can
+ * transform themselves into string representations.  This can be used
+ * together with the string class for automatic conversions.
+ * @author David Sugar <dyfet@gnutelephony.org>
+ */
+class __EXPORT StringFormat
+{
+protected:
+	friend class string;
+	friend class string::cstring;
+	/**
+	 * Convert derived object into a string buffer.  This may be a public
+	 * method in a derived class.
+	 * @param buffer to save representation in.
+	 * @param size of buffer to use.
+	 */
+	virtual void put(char *buffer, size_t size) const = 0;
+
+	/**
+	 * Get maximum space that might be needed for string representation.
+	 * @return space needed.
+	 */
+	virtual strsize_t getStringSize(void) const = 0;
+};
+
+
 /**
  * A template to create a character array that can be manipulated as a string.
  * This is a mini string/stringbuf class that supports a subset of
@@ -1511,7 +1644,7 @@ public:
  * @return 0 if equal, > 0 if s2 > s1, < 0 if s2 < s1.
  */
 extern "C" inline int stricmp(const char *string1, const char *string2)
-	{return string::icompare(string1, string2);};
+	{return string::case_compare(string1, string2);};
 
 /**
  * Convenience function for case insensitive null terminated string compare.
@@ -1521,7 +1654,7 @@ extern "C" inline int stricmp(const char *string1, const char *string2)
  * @return 0 if equal, > 0 if s2 > s1, < 0 if s2 < s1.
  */
 extern "C" inline int strnicmp(const char *string1, const char *string2, size_t max)
-	{return string::icompare(string1, string2, max);};
+	{return string::case_compare(string1, string2, max);};
 
 #endif
 
