@@ -140,7 +140,6 @@ static void socket_mapping(int family, socket_t so)
 #endif
 }
 
-
 #ifndef	HAVE_GETADDRINFO
 
 static mutex servmutex, hostmutex;
@@ -645,6 +644,28 @@ cidr *cidr::find(policy *policy, const struct sockaddr *s)
 	return member;
 }
 
+cidr *cidr::container(policy *policy, const struct sockaddr *s)
+{
+	assert(policy != NULL);
+	assert(s != NULL);
+
+	cidr *member = NULL;
+	unsigned top = 128;
+
+	linked_pointer<cidr> p = policy;
+	while(p) {
+		if(p->isMember(s)) {
+			if(p->getMask() < top) {
+				top = p->getMask();
+				member = *p;
+			}
+		}
+		p.next();
+	}
+	return member;
+}
+
+
 bool cidr::isMember(const struct sockaddr *s) const
 {
 	assert(s != NULL);
@@ -2298,8 +2319,6 @@ switch(addr->sa_family) {
 	return key % keysize;	
 }
 
-
-
 unsigned Socket::keyindex(struct sockaddr *addr, unsigned keysize)
 {
 	assert(addr != NULL);
@@ -2661,9 +2680,13 @@ bool Socket::isNull(const char *str)
 {
 	assert(str != NULL);
 
-	while(*str && strchr("0:.", *str) != NULL)
+	while(*str && strchr("0:.*", *str) != NULL)
 		++str;
-	
+
+	// allow field separation...	
+	if(*str <= ' ')
+		return true;
+
 	if(*str)
 		return false;
 
@@ -2680,7 +2703,11 @@ bool Socket::isNumeric(const char *str)
 
 	while(*str && strchr("0123456789.", *str) != NULL)
 		++str;
-	
+
+	// allow field separators	
+	if(*str <= ' ')
+		return true;
+
 	if(*str)
 		return false;
 
