@@ -17,6 +17,7 @@
 #include <config.h>
 #include <ucommon/string.h>
 #include <ucommon/thread.h>
+#include <ucommon/socket.h>
 #include <stdarg.h>
 #include <ctype.h>
 #include <stdio.h>
@@ -151,7 +152,7 @@ void string::cstring::add(char ch)
 	text[len++] = ch;	
 	fix();
 }
-
+	
 void string::cstring::set(strsize_t offset, const char *str, strsize_t size)
 {
 	assert(str != NULL);
@@ -202,6 +203,24 @@ strsize_t string::len(void)
 	return str->len;
 }
 
+void string::cstring::add(const StringFormat& format)
+{
+	unfix();
+	if(len < max - 1)
+		format.put(text + len, max - len);
+	text[max - 1] = 0;
+	len = strlen(text);
+	fix();
+}
+
+void string::cstring::set(const StringFormat& format)
+{
+	format.put(text, max);
+	text[max - 1] = 0;
+	len = strlen(text);
+	fix();
+}
+
 void string::cstring::set(const char *str)
 {
 	assert(str != NULL);
@@ -248,6 +267,13 @@ string::string(const char *s)
 	str->set(s);
 }
 
+string::string(const StringFormat& f)
+{
+	str = create(f.getStringSize());
+	str->retain();
+	str->set(f);
+}
+
 string::string(const char *s, strsize_t size)
 {
 	if(!s)
@@ -348,6 +374,11 @@ const char *string::c_str(void) const
 	return str->text;
 }
 
+bool string::equal(const char *s) const
+{
+	return compare(s) == 0;
+}
+
 int string::compare(const char *s) const
 {
 	const char *mystr = "";
@@ -750,6 +781,13 @@ void string::rset(const char *s, char overflow, strsize_t offset, strsize_t size
 		str->text[offset] = overflow;
 }
 
+void string::set(const StringFormat& f)
+{
+	resize(f.getStringSize());
+	assert(str);
+	str->set(f);
+}
+
 void string::set(const char *s)
 {
 	strsize_t len;
@@ -795,7 +833,11 @@ bool string::resize(strsize_t size)
 		return true;
 	}
 
-	if(str->isCopied() || str->max < size) {
+	if(!str) {
+		str = create(size, fill);
+		str->retain();
+	}
+	else if(str->isCopied() || str->max < size) {
 		fill = str->fill;
 		str->release();
 		str = create(size, fill);
@@ -843,6 +885,14 @@ void string::cow(strsize_t size)
 	}
 }
 
+void string::add(const StringFormat& f)
+{
+	if(!str)
+		set(f);
+	else
+		str->add(f);
+}
+
 void string::add(const char *s)
 {
 	if(!s || !*s)
@@ -954,6 +1004,19 @@ string &string::operator^=(const char *s)
 	return *this;
 }
 
+string &string::operator^=(const StringFormat& f)
+{
+	release();
+	set(f);
+	return *this;
+}
+
+string &string::operator=(const StringFormat& f)
+{
+	set(f);
+	return *this;
+}
+
 string &string::operator=(const char *s)
 {
 	set(s);
@@ -1052,12 +1115,25 @@ bool string::operator>=(const char *s) const
     return (compare(s) >= 0);
 }
 
+string &string::operator&(const StringFormat& f)
+{
+	add(f);
+	return *this;
+}
+
 string &string::operator&(const char *s)
 {
 	add(s);
 	return *this;
 }
 
+string &string::operator+(const StringFormat& f)
+{
+	cow(f.getStringSize());
+	add(f);
+	return *this;
+}
+
 string &string::operator+(const char *s)
 {
 	if(!s || !*s)
@@ -1677,6 +1753,26 @@ char *string::first(char *str, const char *clist)
     return fp;
 }
 
+bool string::case_equal(const char *s1, const char *s2)
+{
+	return case_compare(s1, s2) == 0;
+}
+
+bool string::equal(const char *s1, const char *s2)
+{
+	return compare(s1, s2) == 0;
+}
+
+bool string::case_equal(const char *s1, const char *s2, size_t size)
+{
+	return case_compare(s1, s2, size) == 0;
+}
+
+bool string::equal(const char *s1, const char *s2, size_t size)
+{
+	return compare(s1, s2, size) == 0;
+}
+
 int string::compare(const char *s1, const char *s2)
 {
 	if(!s1)
@@ -1697,7 +1793,7 @@ int string::compare(const char *s1, const char *s2, size_t s)
     return strncmp(s1, s2, s);
 }
 
-int string::icompare(const char *s1, const char *s2)
+int string::case_compare(const char *s1, const char *s2)
 {
 	if(!s1)
 		s1 = "";
@@ -1712,7 +1808,7 @@ int string::icompare(const char *s1, const char *s2)
 #endif
 }
 
-int string::icompare(const char *s1, const char *s2, size_t s)
+int string::case_compare(const char *s1, const char *s2, size_t s)
 {
     if(!s1)
         s1 = "";
